package com.tfc.efficientroutes;

import java.io.IOException;
import java.util.List;

import android.content.Context;
import android.location.Address;
import android.location.Geocoder;
import android.os.AsyncTask;
import android.widget.Toast;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.LatLngBounds;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;

// Una clase heredada de AsyncTask para acceder al servicio web de GeoCoding
public class BuscaDireccio extends AsyncTask<String, Void, List<Address>>{

	private GoogleMap mapa;
	private Context context;
	private LatLng latLng;
	private MarkerOptions markerOptions;
	private List<Marker> markersList;
	
	BuscaDireccio(GoogleMap mapa, Context context, LatLng latLng, MarkerOptions markerOptions, List<Marker> markersList) {
		this.mapa = mapa;
		this.context = context;
		this.latLng = latLng;
		this.markerOptions = markerOptions;
		this.markersList = markersList;
	}
	
    @Override
    protected List<Address> doInBackground(String... locationName) {
        // Creamos una instancia de la clase Geocoder
        Geocoder geocoder = new Geocoder(context);
        List<Address> addresses = null;

        try {
            // Limitamos a 4 el resultado mximo de direcciones coincidientes en la bsqueda
            addresses = geocoder.getFromLocationName(locationName[0], 4);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return addresses;
    }

    @Override
    protected void onPostExecute(List<Address> addresses) {

    	// Limpia todos los markers que haya en el mapa
        ActualitzaMapa.actualitzaMapa(mapa, markersList);
        
        if(addresses==null || addresses.size()==0){
            Toast.makeText(context, "No hay resultados", Toast.LENGTH_SHORT).show(); 
            
        } else {
        	
        	double latMax = -90;
            double latMin = 90;
            double lngMax = -180;
            double lngMin = 180;
            double lat, lng;
            
            // Aadimos Markers en el mapa para cada direccin encontrada
            int i = 0;
            while (i < addresses.size()) {                	
 
                Address address = (Address) addresses.get(i);
                
                lat = address.getLatitude();
                lng = address.getLongitude();
                
                if (i==0) {
                	latMax = latMin = lat;
                	lngMax = lngMin = lng;
                } else {
                	if (lat > latMax) latMax = lat;
                    else if (lat < latMin) latMin = lat;
                    if (lng > lngMax) lngMax = lng;
                    else if (lng < lngMin) lngMin = lng;
                }
                i++;

                // Creamos una instancia de GeoPoint para mostrarla en el mapa
                latLng = new LatLng(lat, lng);
 
                String addressText = String.format("%s, %s",
                address.getMaxAddressLineIndex() > 0 ? address.getAddressLine(0) : "",
                address.getLocality() != null ? address.getLocality() : "");
 
                markerOptions = new MarkerOptions();
                markerOptions.position(latLng);
                markerOptions.title(addressText);
 
                mapa.addMarker(markerOptions);

            }
            if (i == 1) mapa.animateCamera(CameraUpdateFactory.newLatLngZoom(latLng, 15));
            else {
            	LatLngBounds ltlgbounds = new LatLngBounds(new LatLng(latMin, lngMin), new LatLng(latMax, lngMax));
            	mapa.animateCamera(CameraUpdateFactory.newLatLngBounds(ltlgbounds, 150));
            }
            
        }

    }
}
