package com.tfc.efficientroutes;


import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ToggleButton;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesUtil;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.GoogleMap.OnInfoWindowClickListener;
import com.google.android.gms.maps.GoogleMap.OnMapClickListener;
import com.google.android.gms.maps.GoogleMap.OnMarkerClickListener;
import com.google.android.gms.maps.GoogleMap.OnMyLocationChangeListener;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;

public class MainActivity extends FragmentActivity implements OnMyLocationChangeListener {

	private GoogleMap mapa = null;
	private int vista = 0;
	private MarkerOptions markerOptions;
	private LatLng pos_actual = null;
	private LatLng latLng;
	private EditText et_localizaciones;
	private TextView tvDistanceDuration;
	private List<Marker> markersList = new ArrayList<Marker>();
	private boolean posActualOrigen = true;
	private boolean posActualDestino = true;
	private Button btn_buscar;    
    private Button btn_optimizar;
    private Button btn_borrar;
	private ToggleButton toggleBtn_coche; 
	private ToggleButton toggleBtn_bici;
	private ToggleButton toggleBtn_pie;
	private ProgressDialog dialog;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
        // Getting Google Play availability status
        int status = GooglePlayServicesUtil.isGooglePlayServicesAvailable(getBaseContext());
 
        // Showing status
        if(status!=ConnectionResult.SUCCESS){ // Google Play Services are not available
            int requestCode = 10;
            Dialog dialog = GooglePlayServicesUtil.getErrorDialog(status, this, requestCode);
            dialog.show();
 
        }else {    // Google Play Services are available
 
            // Getting reference to the SupportMapFragment of activity_main.xml
            SupportMapFragment fm = (SupportMapFragment) getSupportFragmentManager().findFragmentById(R.id.map);
 
            // Getting GoogleMap object from the fragment
            mapa = fm.getMap();
            
            // Showing the location in Google Map
            mapa.moveCamera(CameraUpdateFactory.newLatLngZoom(new LatLng(41.391491,2.174209), 4));
            
            // Referenciamos los componentes con sus objetos
			tvDistanceDuration = (TextView) findViewById(R.id.tv_distancia_tiempo);
	        et_localizaciones = (EditText) findViewById(R.id.et_localizacion);
			
	        btn_buscar = (Button) findViewById(R.id.btn_buscar);    
	        btn_optimizar = (Button) findViewById(R.id.btn_optimizar);
	        btn_borrar = (Button) findViewById(R.id.btn_borrar);

	        toggleBtn_coche = (ToggleButton) findViewById(R.id.toggleBtn_coche);
     		toggleBtn_bici = (ToggleButton) findViewById(R.id.toggleBtn_bici);
     		toggleBtn_pie = (ToggleButton) findViewById(R.id.toggleBtn_pie);
	        
     		// Asignamos a los listeners a los componentes
            mapa.setMyLocationEnabled(true);
            mapa.setOnMyLocationChangeListener(this);
            mapa.setOnMapClickListener(clickEnMapa);
	        mapa.setOnMarkerClickListener(clickEnMarcador);
	        mapa.setOnInfoWindowClickListener(clickInfoMarcador);
	 
	        btn_buscar.setOnClickListener(clickEnBuscar);
	        btn_optimizar.setOnClickListener(clickEnOptimitzar);
	        btn_borrar.setOnClickListener(clickEnEsborrar);

     		toggleBtn_coche.setOnCheckedChangeListener(canviToggleButton);
     		toggleBtn_bici.setOnCheckedChangeListener(canviToggleButton);
     		toggleBtn_pie.setOnCheckedChangeListener(canviToggleButton);
	 		
     		// Creamos el mensaje emergente de espera
			dialog = new ProgressDialog(this);
	        dialog.setMessage("Calculando la ruta ptima");
	        dialog.setTitle("Cargando...");
	        dialog.setCancelable(true);
	    }
	}
	
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}
	
	
  	@Override
    public void onMyLocationChange(Location location) {
 
        // Getting latitude of the current location
        double latitude = location.getLatitude();
 
        // Getting longitude of the current location
        double longitude = location.getLongitude();
 
        // Creating a LatLng object for the current location
        pos_actual = new LatLng(latitude, longitude);
        
    }
	
	
	// Selector de men desplegable
	@Override
	public boolean onOptionsItemSelected(MenuItem item) 
	{	
		switch(item.getItemId())
		{
			// Una sola opcin, para cambiar el tipo de vista del mapa
			case R.id.menu_vista:
				alternarVista();
				break;
				
			case R.id.menu_origen:
				if(posActualOrigen) {
					posActualOrigen = false;
					item.setChecked(false);
				}
				else {
					posActualOrigen = true;
					item.setChecked(true);
				}
				break;
				
			case R.id.menu_destino:
				if(posActualDestino) {
					posActualDestino = false;
					item.setChecked(false);
				}
				else {
					posActualDestino = true;
					item.setChecked(true);
				}
				break;
		}

		return super.onOptionsItemSelected(item);
	}
	
	
	// Definimos el listener del evento click a botn 'Buscar'
    OnClickListener clickEnBuscar = new OnClickListener() {
        @Override
        public void onClick(View v) {
            
            // Obtenemos la localizacin
            String localizacion = et_localizaciones.getText().toString();

            if(localizacion!=null && !localizacion.equals("")){
                new BuscaDireccio(mapa, getBaseContext(), latLng, markerOptions, markersList).execute(localizacion);
                
                // Ocultamos el teclado para ver los Markers claramente
      			InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
      			imm.hideSoftInputFromWindow(et_localizaciones.getWindowToken(), 0);
            }
        }
    };
    

    // Definimos el listener del evento click a botn 'Optimizar'
    OnClickListener clickEnOptimitzar = new OnClickListener() {
        @Override
        public void onClick(View v) {
        	if(pos_actual == null) {
                Toast.makeText(getBaseContext(), "Espera a obtener tu ubicacin", Toast.LENGTH_SHORT).show(); 
        	} else if (markersList.isEmpty()){
    			Toast.makeText(getBaseContext(), "No hay destinos seleccionados", Toast.LENGTH_SHORT).show();
        	} else	{
        		// Limpia todos los markers que haya en el mapa
                ActualitzaMapa.actualitzaMapa(mapa, markersList);
               
        		optimizaRuta(markersList);
        	}
        	
        }
    };
    
    
    // Definimos el listener del evento click a botn 'Borrar'
    OnClickListener clickEnEsborrar = new OnClickListener() {
        @Override
        public void onClick(View v) {
        	markersList = new ArrayList<Marker>();
        	mapa.clear();
        	tvDistanceDuration.setText(R.string.tv_distancia_tiempo);
        }
    };
    
	
 	OnCheckedChangeListener canviToggleButton = new OnCheckedChangeListener() {
 	    @Override
 	    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
 	        if (isChecked){
 	            if (buttonView == toggleBtn_coche) {
 	            	toggleBtn_bici.setChecked(false);
 	            	toggleBtn_pie.setChecked(false);
 	            }
 	            if (buttonView == toggleBtn_bici) {
 	            	toggleBtn_coche.setChecked(false);
 	            	toggleBtn_pie.setChecked(false);
 	            }
 	            if (buttonView == toggleBtn_pie) {
 	            	toggleBtn_bici.setChecked(false);
 	            	toggleBtn_coche.setChecked(false);
 	            }
 	        } else {
 	        	if (buttonView == toggleBtn_coche && !toggleBtn_bici.isChecked() && !toggleBtn_pie.isChecked()) {
 	        		 toggleBtn_coche.setChecked(true);
 	        	}
 	        	if (buttonView == toggleBtn_bici && !toggleBtn_pie.isChecked() && !toggleBtn_coche.isChecked()) {
	        		 toggleBtn_bici.setChecked(true);
	        	}
 	        	if (buttonView == toggleBtn_pie && !toggleBtn_coche.isChecked() && !toggleBtn_bici.isChecked()) {
	        		 toggleBtn_pie.setChecked(true);
	        	}
 	        }
 	    }
 	};
 	
    
	OnMapClickListener clickEnMapa = new OnMapClickListener() {
		@Override
		public void onMapClick(LatLng point) {		
			Geocoder geo = new Geocoder(getBaseContext());
			List<Address> addresses = null;
			try {
				addresses = geo.getFromLocation(point.latitude, point.longitude, 1);
			} catch (IOException e) {
                e.printStackTrace();
            }
            
			if (addresses.isEmpty()) {
				Toast.makeText(getBaseContext(), "No es un punto vlido", Toast.LENGTH_SHORT).show();
				return;
			}
			Address address = addresses.get(0);
			
			String addressText = String.format("%s, %s",
                    address.getMaxAddressLineIndex() > 0 ? address.getAddressLine(0) : "",
                    address.getLocality() != null ? address.getLocality() : "");
     
			// Limpia todos los markers que haya en el mapa
	        ActualitzaMapa.actualitzaMapa(mapa, markersList);
			
			// Creating and add new marker to the Google Map Android API V2
            Marker marker = mapa.addMarker(new MarkerOptions()
                // Setting the position of the marker
                .position(point)
                .title(addressText));
			marker.showInfoWindow();
		}
	}; 
    
    
    // Asignamos al mapa su listener de click en Markers
    OnMarkerClickListener clickEnMarcador = new OnMarkerClickListener() {
    	@Override
		public boolean onMarkerClick(Marker marker) {
			Geocoder geo = new Geocoder(getBaseContext());
			List<Address> address = null;
			try {
				address = geo.getFromLocation(marker.getPosition().latitude, marker.getPosition().longitude, 1);
			} catch (IOException e) {
                e.printStackTrace();
            }
            
			if (address.isEmpty()) {
				Toast.makeText(getBaseContext(), "No es un punto vlido", Toast.LENGTH_SHORT).show();
				return true;
			}
    		
    		marker.showInfoWindow();
    		return true;
    	}
    };
    
    
    OnInfoWindowClickListener clickInfoMarcador = new OnInfoWindowClickListener() {
		@Override
		public void onInfoWindowClick(Marker marker) {
			marker.setIcon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_GREEN));
			markersList.add(marker);
    		
			if (et_localizaciones!=null)
				et_localizaciones.setText(null);
		}
    };
 	
 	

    
	
	// Mtodo para alternar los tipos de mapas de Google Maps
	private void alternarVista() {
	    vista = (vista + 1) % 4;
	 
	    switch(vista) {
	        case 0:
	            mapa.setMapType(GoogleMap.MAP_TYPE_NORMAL);	// Vista de mapa callejero
	            break;
	        case 1:
	            mapa.setMapType(GoogleMap.MAP_TYPE_SATELLITE); // Vista de satlite
	            break;
	        case 2:
	            mapa.setMapType(GoogleMap.MAP_TYPE_TERRAIN); // Vista de relieve
	            break;
	        case 3:
	            mapa.setMapType(GoogleMap.MAP_TYPE_HYBRID); // Vista de mapa callejero con satlite
	            break;

	    }
	}
		
    
    // Funcin que devuelve una URL montada para enviar a servicio web de Google
	private String getDirectionsUrl(LatLng origen,LatLng destino, List<LatLng> waypoints){
		
		// Origen de la ruta
		String str_origen = "origin=" + origen.latitude + "," + origen.longitude;
		
		// Destino de la ruta
		String str_destino = "destination=";
		if (destino != null) str_destino = str_destino + destino.latitude + "," + destino.longitude;		
					
		// Sensor activado
		String sensor = "sensor=false";	
		
		String region = "region=es";
		
		String str_waypoints = "waypoints=optimize:true";
		for (int i = 0; i < waypoints.size(); i++) {
			str_waypoints = str_waypoints + "|" + waypoints.get(i).latitude + "," + waypoints.get(i).longitude;
		}
		
		// Travelling Mode
		String mode = "mode=driving";	
		
		if(toggleBtn_bici.isChecked()){
			mode = "mode=bicycling";
			//mMode = 1;
		}else if(toggleBtn_pie.isChecked()){
			mode = "mode=walking";
			//mMode = 2;
		}
					
		// Building the parameters to the web service
		String parametros = str_origen + "&" + str_destino + "&" + str_waypoints + "&" + sensor+ "&" + region + "&" + mode;
					
		// Formato de salida (JSON/XML)
		String salida = "json";
		
		
		// Montamos la URL para el servicio web
		String url = "https://maps.googleapis.com/maps/api/directions/"+salida+"?"+parametros;
		
		//https://maps.googleapis.com/maps/api/directions/json?origin=41.367219,2.096102&destination=41.391137,2.188713&waypoints=optimize:true|41.384613,2.113682|41.355165,2.123695|41.387871,2.168166&sensor=false
		return url;
	}
    
        	
  	private void optimizaRuta(List<Marker> markersList) {
  		LatLng origen;
		LatLng destino;
		
		if (markersList.size() == 1 && !posActualOrigen && !posActualDestino) {
			Toast.makeText(getBaseContext(), "Debe haber al menos dos localizaciones", Toast.LENGTH_SHORT).show();
			return;
		}
		
		if (posActualOrigen) {
			origen = pos_actual;
		} else {
			origen = markersList.get(0).getPosition();
		}
		
		
		if (posActualDestino) {
			destino = pos_actual;
		} else {
			destino = null;
		}

		List<LatLng> waypointsList = new ArrayList<LatLng>();
		for (int i = 0; i < markersList.size(); i++) {
			waypointsList.add(markersList.get(i).getPosition());
		}
		// Montar la URL para la API de Google Directions
		String url = getDirectionsUrl(origen, destino, waypointsList);				
		
		CarregarURL downloadTask = new CarregarURL(markersList, getBaseContext(), mapa, tvDistanceDuration, dialog);
		
		// Recibimos datos JSON de la API de Google Directions
		downloadTask.execute(url);
	}
  	
}
